/**********************************************************************
 *
 * browsetools.h -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: browsetools.h 344 1999-07-07 05:44:25Z sjboddie $
 *
 *********************************************************************/


#ifndef BROWSETOOLS_H
#define BROWSETOOLS_H

#include "gsdlconf.h"
#include "text_t.h"
#include "cgiargs.h"
#include "display.h"
#include "recptproto.h"


// Note that at present the build software is capable (if you write a plugin to do
// it) of generating classifications that contain other classifications of a 
// different type. This module can only handle one case of that however (i.e. an
// AZList containing a Hierarchy classification as used by hdl's 'magazine'
// section). It really needs redesigning to handle all cases.


/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
Functions for generating a "Book" type table of contents. These should only be called
for document level tocs (i.e. when the "d" argument is set) as I don't think it makes 
sense to display top level classifications in this way.
This type of table of contents is intended for documents whose sub-sections are simply 
numbered as pages.
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void book_toc (const text_t &/*classifytitle*/, cgiargsclass &args, 
	       recptproto *collectproto, displayclass &disp, 
	       outconvertclass &outconvert, ostream &textout, 
	       ostream &logout);


/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
Functions for generating a "Hierarchy" type table of contents. These can be used either
at top classification level or at document level (the difference being that a cover
image and control buttons are displayed at document level).
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void hierarchy_toc (const text_t &classifytype, const text_t &classifytitle, 
		    cgiargsclass &args, recptproto *collectproto, 
		    displayclass &disp, outconvertclass &outconvert, 
		    ostream &textout, ostream &logout);
void output_hierarchy (const text_t &classifytype, cgiargsclass &args, 
		       recptproto *collectproto, displayclass &disp, 
		       outconvertclass &outconvert, ostream &textout, 
		       ostream &logout);
void output_contracted_hierarchy_toc (const text_t &classifytype, cgiargsclass &args, 
				      recptproto *collectproto,displayclass &disp, 
				      outconvertclass &outconvert, ostream &textout, 
				      ostream &logout);
void output_expanded_hierarchy_toc (cgiargsclass &args, recptproto *collectproto,
				    displayclass &disp, outconvertclass &outconvert,
				    ostream &textout, ostream &logout);
void output_parents_toc (cgiargsclass &args, const text_tarray &parents,
			 int &tabcount, int &colnum, displayclass &disp,
			 recptproto *collectproto, outconvertclass &outconvert,
			 ostream &textout, ostream &logout);
void output_siblings_toc (cgiargsclass &args, const FilterResponse_t &siblings, 
			  int &tabcount, int &colnum, displayclass &disp, 
			  outconvertclass &outconvert, ostream &textout, ostream &logout);
void output_controls (cgiargsclass &args, recptproto *collectproto, 
		      displayclass &disp, outconvertclass &outconvert, 
		      ostream &textout, ostream &logout);
void output_cover_image (cgiargsclass &args, recptproto *collectproto, 
			 displayclass &disp, outconvertclass &outconvert, 
			 ostream &textout, ostream &logout);



/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
Functions for generating a "List" type table of contents. These can be used only
at classification level as it doesn't make sense to display document level stuff
this way (I think!!)
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void list_toc (const text_t &classifytype, const text_t &classifytitle, 
	       cgiargsclass &args, recptproto *collectproto, 
	       displayclass &disp, outconvertclass &outconvert, 
	       ostream &textout, ostream &logout);


/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
Functions for generating an "AZList" type table of contents. These can be used only
at classification level as it doesn't make sense to display document level stuff
this way (I think!!)
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void azlist_toc (const text_t &classifytype, const text_t &classifytitle, 
		 cgiargsclass &args, recptproto *collectproto, 
		 displayclass &disp, outconvertclass &outconvert, 
		 ostream &textout, ostream &logout);
void output_azlist (const FilterResponse_t &response, const text_t &classifytitle,
		    const text_t &arg_cl, displayclass &disp, 
		    outconvertclass &outconvert, ostream &textout);


/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
Functions for generating a "DateList" type table of contents. These can be used only
at classification level as it doesn't make sense to display document level stuff
this way (I think!!)
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void datelist_toc (const text_t &classifytype, const text_t &classifytitle, 
		   cgiargsclass &args, recptproto *collectproto, 
		   displayclass &disp, outconvertclass &outconvert, 
		   ostream &textout, ostream &logout);


/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void set_arrow_macros (const text_t &OID, const text_t &classifytype,
		       displayclass &disp, recptproto *collectproto, 
		       const text_t &collection, ostream &logout);




#endif
